using System;
using System.IO;
using System.Linq;
using FMSAdmin.Data;
using FMSAdmin.Entities;
using FMSAdmin.Helpers;
using FMSAdmin.Models;
using FMSAdmin.Services;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using OfficeOpenXml;

namespace FMSAdmin.Controllers {
    [Authorize]
    [ApiController]
    [ApiVersion("1")]
    [Route("api/[controller]")]
    public class AlarmPointController : Controller {
        private readonly ILogger<AlarmPointController> _logger;
        private readonly FMSContext _context;
        private AlarmPointService _service;

        public AlarmPointController(
            ILogger<AlarmPointController> logger,
            FMSContext context,
            AlarmPointService service
        ) {
            _logger = logger;
            _context = context;
            _service = service;
        }

        /// <summary>
        /// 검색 & 정렬
        /// </summary>
        /// <param name="req"></param>
        /// <returns></returns>
        private IQueryable<FmsAlarmPoint> _FilterAndSort(PagingRequest req) {
            var query = _service.GetAll();
            query = query.Filter(req.conditions);
            if (req.siteId != null) {
                query = query.Where(x => x.SiteId == Util.ToInt(req.siteId));
            }
            if (req.siteIds != null && req.siteIds.Length > 0) {
                query = query.Where(x => req.siteIds.Contains(x.SiteId));
            }
            query = query.Sort(req.sort);
            return query;
        }


        /// <summary>
        /// 셀렉트 공통
        /// </summary>
        /// <param name="query"></param>
        /// <returns></returns>
        private dynamic _ListSelect(IQueryable<FmsAlarmPoint> query) {
            var list = query.Select(x => new {
                x.SiteId,
                SiteName = x.Site.Name,
                x.FacilityTypeId,
                FacilityTypeName = x.FacilityType.Name,
                x.FacilityCode,
                FacilityName = x.Facility.Name,
                PointName = x.BemsMonitoringPoint.Name,
                x.PropertyId,
            });
            return list;
        }
        /// <summary>
        /// 목록
        /// </summary>
        /// <param name="req"></param>
        /// <param name="siteId"></param>
        /// <returns></returns>
        [HttpGet]
        public IActionResult List([FromQuery] PagingRequest req) {
            var query = _FilterAndSort(req);
            var list = _ListSelect(query);
            var paging = PagingList.Pagination(list, req.page, req.limit);

            return Ok(paging);
        }

        // 엑셀 다운로드
        [AllowAnonymous]
        [HttpGet("excel")]
        public IActionResult ExportExcel([FromQuery] PagingRequest req, string filename) {
            var query = _FilterAndSort(req);
            var list = _ListSelect(query);

            var stream = new MemoryStream();
            using (var package = new ExcelPackage(stream)) {
                var workSheet = package.Workbook.Worksheets.Add("Sheet1");

                workSheet.Cells.LoadFromCollection(list, true);

                if (req.columns != null && req.columns.Length > 0) {
                    workSheet.AddStyle(req.columns, req.sort?.order?.ToLower() == "asc", filename);
                }
                package.Save();
            }
            stream.Position = 0;
            string excelName = $"excel-{DateTime.Now.ToString("yyyyMMddHHmmssfff")}.xlsx";
            return File(stream, "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", excelName);
        }

        /// <summary>
        /// 조회
        /// </summary>
        [HttpGet("{siteId}/{facilityCode}/{propertyId}")]
        public IActionResult Get(int siteId, int facilityCode, int propertyId) {
            var data = _service.Get(siteId, facilityCode, propertyId);
            if (data.Count() == 0) {
                return NotFound("사이트 정보를 찾을 수 없습니다.");
            }
            var list = data.Select(x => new {
                x.SiteId,
                CmSite = new {
                    x.Site.Name,
                },
                x.FacilityCode,
                Facility = new {
                    x.Facility.Name,
                    TypeName = x.Facility.BemsFacilityType.Name
                },
                BemsMonitoringPoint = new {
                    x.BemsMonitoringPoint.Name,
                },
                ServiceType = new {
                    x.BemsMonitoringPoint.ServiceType.Name
                },
                x.PropertyId,
                x.FacilityTypeId,
                FuelType = new {
                    x.BemsMonitoringPoint.FuelType.Name
                }
            });
            return Ok(list.First());
        }


        /// <summary>
        /// 등록
        /// </summary>
        [HttpPut]
        public IActionResult Create([FromBody] FmsAlarmPoint[] data) {
            _logger.LogInformation("Create");
            // _logger.LogInformation(data.Dump());

            if (ModelState.IsValid) {
                _service.Create(data);
            } else {
                return BadRequest(ModelState);
            }
            return Ok();
        }

        /// <summary>
        /// 삭제
        /// </summary>
        [HttpDelete("{siteId}/{facilityCode}/{propertyId}")]
        public IActionResult Delete(int siteId, int facilityCode, int propertyId) {
            _service.Delete(siteId, facilityCode, propertyId);
            return Ok();
        }

    }
}