using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using FMSAdmin.Data;
using FMSAdmin.Entities;
using FMSAdmin.Helpers;
using FMSAdmin.Models;
using FMSAdmin.Services;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using OfficeOpenXml;

namespace FMSAdmin.Controllers {
    [Authorize]
    [ApiController]
    [ApiVersion("1")]
    [Route("api/[controller]")]
    public class LayoutController : Controller {
        private readonly ILogger<LayoutController> _logger;
        private readonly FMSContext _context;
        private LayoutService _service;
        private readonly StorageHelper _storage; public LayoutController(
           ILogger<LayoutController> logger,
           FMSContext context,
           LayoutService service,
            StorageHelper storage
       ) {
            _logger = logger;
            _context = context;
            _service = service;
            _storage = storage;
        }

        /// <summary>
        /// /// 검색 & 정렬
        /// </summary>
        /// <param name="req"></param>
        /// <returns></returns>
        private IQueryable<CmLayout> _FilterAndSort(PagingRequest req) {
            var query = _service.GetAll();
            query = query.Filter(req.conditions);
            if (req.siteId != null) {
                query = query.Where(x => x.SiteId == Util.ToInt(req.siteId));
            }
            if (req.siteIds != null && req.siteIds.Length > 0) {
                query = query.Where(x => req.siteIds.Contains(x.SiteId));
            }
            // 고정정렬 (월->사옥 준공순)

            var sortQuery = query.OrderByDescending(x => x.Year);
            sortQuery = sortQuery.ThenByDescending(x => x.Month);
            sortQuery = sortQuery.ThenBy(x => x.CmSite.SortOrderNo);
            sortQuery = sortQuery.ThenSort(req.sort);
            return sortQuery;
        }


        /// <summary>
        /// 셀렉트 공통
        /// </summary>
        /// <param name="query"></param>
        /// <returns></returns>
        private dynamic _ListSelect(IQueryable<CmLayout> query) {
            var list = query.Select(x => new {
                x.LayoutId,
                x.SiteId,
                SiteName = x.CmSite.Name,
                x.Year,
                x.Month,
                Title = x.Year.ToString() + "년 " + x.Month.ToString() + "월 레이아웃현황",
                FilePath = _storage.getFilePath(x.CmFile),
                FileName = x.CmFile.Name,
                FilePath2 = _storage.getFilePath(x.CmFile2),
                FileName2 = x.CmFile2.Name
            });
            return list;
        }
        /// <summary>
        /// 목록
        /// </summary>
        /// <param name="req"></param>
        /// <param name="siteId"></param>
        /// <returns></returns>
        [HttpGet]
        public IActionResult List([FromQuery] PagingRequest req) {
            var query = _FilterAndSort(req);
            var list = _ListSelect(query);
            var paging = PagingList.Pagination(list, req.page, req.limit);

            return Ok(paging);
        }

        // 엑셀 다운로드
        [AllowAnonymous]
        [HttpGet("excel")]
        public IActionResult ExportExcel([FromQuery] PagingRequest req, string filename) {
            var query = _FilterAndSort(req);
            var list = _ListSelect(query);

            var stream = new MemoryStream();
            using (var package = new ExcelPackage(stream)) {
                var workSheet = package.Workbook.Worksheets.Add("Sheet1");

                workSheet.Cells.LoadFromCollection(list, true);

                if (req.columns != null && req.columns.Length > 0) {
                    workSheet.AddStyle(req.columns, req.sort?.order?.ToLower() == "asc", filename);
                }
                package.Save();
            }
            stream.Position = 0;
            string excelName = $"excel-{DateTime.Now.ToString("yyyyMMddHHmmssfff")}.xlsx";
            return File(stream, "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", excelName);
        }

        /// <summary>
        /// 조회
        /// </summary>
        [HttpGet("{siteId}/{id}")]
        public IActionResult Get(int siteId, int id) {
            var data = _service.Get(id, siteId);
            if (data.Count() == 0) {
                return NotFound("사이트 정보를 찾을 수 없습니다.");
            }
            var list = data.Select(x => new {
                x.LayoutId,
                x.SiteId,
                CmSite = new {
                    x.CmSite.Name,
                },
                x.Year,
                x.Month,
                CmUser = new { x.CmUser.Name },
                CmFile = _storage.ParseFile(x.CmFile),
                CmFile2 = _storage.ParseFile(x.CmFile2),
            });
            return Ok(list.First());
        }

        /// <summary>
        /// 수정
        /// </summary>
        [HttpPost("{siteId}/{id}")]
        public IActionResult Edit(int siteId, int id, [FromBody] CmLayout data) {
            _logger.LogInformation("Edit");
            _logger.LogInformation("id:" + id);
            _logger.LogInformation("siteId:" + siteId);
            //_logger.LogInformation(data.Dump());

            if (id != data.LayoutId) {
                return BadRequest("잘못된 접근입니다.");
            }
            ModelState.Remove("RegisterUserId");
            if (ModelState.IsValid) {
                data.UpdateUserId = User.Identity.Name;
                _service.Edit(id, siteId, data);
            } else {
                return BadRequest(ModelState);
            }

            return Ok();
        }

        /// <summary>
        /// 등록
        /// </summary>
        [HttpPut]
        public IActionResult Create([FromBody] CmLayout data) {
            _logger.LogInformation("Create");
            //_logger.LogInformation(data.Dump());
            ModelState.Remove("RegisterUserId");
            if (ModelState.IsValid) {
                data.RegisterUserId = User.Identity.Name;
                _service.Create(data);
            } else {
                return BadRequest(ModelState);
            }
            return Ok();
        }

        /// <summary>
        /// 삭제
        /// </summary>
        [HttpDelete("{siteId}/{id}")]
        public IActionResult Delete(int siteId, int id) {
            _service.Delete(id, siteId);
            return Ok();
        }

    }
}