using System;
using System.Linq;
using System.Threading.Tasks;

using FMSAdmin.Data;
using FMSAdmin.Models;
using FMSAdmin.Entities;
using FMSAdmin.Services;

using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Logging;
using Microsoft.AspNetCore.Authorization;
using Microsoft.EntityFrameworkCore;
using FMSAdmin.Helpers;
using System.Data;
using System.IO;
using OfficeOpenXml;
using Newtonsoft.Json;
using Microsoft.AspNetCore.Http;
using System.Collections.Generic;
using System.Collections;

namespace FMSAdmin.Controllers {
    [Authorize]
    [ApiController]
    [ApiVersion("1")]
    [Route("api/[controller]")]
    public class MaterialOrderStatusByRequestController : Controller {
        private readonly ILogger<MaterialOrderStatusByRequestController> _logger;
        private readonly FMSContext _context;
        private readonly MaterialOrderStatusByRequestService _service;
        private readonly StorageHelper _storage;

        public MaterialOrderStatusByRequestController(
            ILogger<MaterialOrderStatusByRequestController> logger,
            FMSContext context,
            MaterialOrderStatusByRequestService service,
            StorageHelper storage
            ) {
            _logger = logger;
            _context = context;
            _service = service;
            _storage = storage;
        }

        /// <summary>
        /// 리스트
        /// </summary>
        [HttpGet]
        public IActionResult List([FromQuery] PagingRequest req) {
            var query = _FilterAndSort(req);

            var list = _ListSelect(query);

            var paging = PagingList.Pagination(list, req.page, req.limit);
            return Ok(paging);
        }

        /// <summary>
        /// 엑셀파일 익스포트
        /// </summary>
        /// <param name="req"></param>
        /// <returns></returns>
        [HttpGet("excel")]
        public IActionResult ExportExcel([FromQuery] PagingRequest req, string filename) {
            req.page = 1;
            req.limit = 0;
            var query = _FilterAndSort(req);

            var list = _ListSelect(query);

            var stream = new MemoryStream();
            using (var package = new ExcelPackage(stream)) {
                var workSheet = package.Workbook.Worksheets.Add("Sheet1");

                workSheet.Cells.LoadFromCollection(list, true);

                if (req.columns != null && req.columns.Length > 0) {
                    workSheet.AddStyle(req.columns, req.sort?.order?.ToLower() == "asc", filename);
                }
                package.Save();
            }
            stream.Position = 0;
            string excelName = $"excel-{DateTime.Now.ToString("yyyyMMddHHmmssfff")}.xlsx";
            return File(stream, "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", excelName);
        }

        /// <summary>
        /// 엑셀파일 임포트
        /// </summary>
        /// <param name="file"></param>
        /// <returns></returns>
        [HttpPost]
        [Route("[action]")]
        public IActionResult ImportExcel(IFormFile file) {
            IList<Hashtable> list = new List<Hashtable>();
            using (ExcelPackage excelPackage = new ExcelPackage(file.OpenReadStream())) {
                var worksheet = excelPackage.Workbook.Worksheets[0];
                for (int i = worksheet.Dimension.Start.Row; i <= worksheet.Dimension.End.Row; i++) {

                    var name = worksheet.Cells[i, 1].Value;
                    var rfid = worksheet.Cells[i, 2].Value;
                    var code = worksheet.Cells[i, 3].Value;
                    list.Add(new Hashtable {
                        {"Name", name}
                    });
                }
            }
            //_service.ImportExcel(list);
            //return BadRequest(ex.Message);

            return Ok();
        }

        /// <summary>
        /// 조회
        /// </summary>
        [HttpGet("{siteId}/{id}")]
        public IActionResult Get(int siteId, int id) {
            var data = _service.Gets(id, siteId);
            if (data.Count() == 0) {
                return NotFound("자재 구매 신청 목록을 찾을 수 없습니다.");
            }

            var item = data.Select(x => new {
                x.PurchaseRequestId,
                x.SiteId,
                CmSite = new {
                    x.Site.Name,
                },
                x.BusinessFieldId,
                CmBusinessField = new {
                    x.CmBusinessField.Name,
                },
                x.PurchaseTypeId,
                PurchaseTypeName = x.PurchaseType.Name,
                x.RequestDate,
                x.RequestUserId,
                RequestUserName = x.CmUserNavigation.Name,
                x.Title,
                x.Reason,
                x.ProgressId,
                ProgressName = x.Progress.Name,
                MaterialPurchaseRequestMaterial = x.FmsMaterialPurchaseRequestMaterial.Select(f => new {
                    f.SiteId,
                    f.BusinessFieldId,
                    f.PurchaseRequestId,
                    f.MaterialId,
                    f.MaterialCount,
                    MaterialName = f.FmsMaterial.Name,
                    MaterialStandard = f.FmsMaterial.Standard,
                    MaterialUnit = f.FmsMaterial.Unit,
                    OrderMaterialCount = _context.FmsMaterialPurchaseOrderMaterial.Where(fo => fo.SiteId == f.SiteId && x.FmsMaterialPurchaseOrder.Select(fp => fp.PurchaseOrderId).Contains(fo.PurchaseOrderId) && fo.MaterialId == f.MaterialId).Sum(fo => fo.MaterialCount),
                    StoredDueDate = x.FmsMaterialPurchaseOrder.Select(fo => fo.StoredDueDate).FirstOrDefault(),
                }).ToList(),
            });

            return Ok(item.First());
        }

        // 셀렉트 공통
        private dynamic _ListSelect(IQueryable<FmsMaterialPurchaseRequest> query) {
            var list = query.Select(x => new {
                x.PurchaseRequestId,
                x.SiteId,
                SiteName = x.Site.Name,
                x.BusinessFieldId,
                BusinessFieldName = x.CmBusinessField.Name,
                ProgressName = x.Progress.Name,
                x.Title,
                RequestDate = (x.RequestDate != null) ? x.RequestDate.ToString("yyyy-MM-dd") : "",
                x.RequestUserId,
                RequestUserName = x.CmUserNavigation.Name,
                ApprovalDate = (x.ApprovalDate != null) ? x.ApprovalDate.Value.ToString("yyyy-MM-dd") : "",
                x.ApprovalUserId,
                ApprovalUserName = x.CmUser.Name,
                x.BSameOrder,
            });
            return list;
        }

        // 검색 & 정렬 공통
        private IQueryable<FmsMaterialPurchaseRequest> _FilterAndSort(PagingRequest req) {
            var query = _service.GetAll();

            // 기본 Entity 검색
            query = query.Filter(req.conditions);

            if (req.siteId != null) {
                query = query.Where(x => x.SiteId == Util.ToInt(req.siteId));
            }
            if (req.siteIds != null && req.siteIds.Length > 0) {
                query = query.Where(x => req.siteIds.Contains(x.SiteId));
            }

            // 업무분야 필드가 있는경우만 추가
            if (req.businessFieldId != null) {
                query = query.Where(x => x.SiteId == req.siteId);
                query = query.Where(x => x.BusinessFieldId == req.businessFieldId || x.CmBusinessField.Name == "공통");
            }

            // 고정정렬 (사옥 준공순)
            var sortQuery = query.OrderBy(x => x.Site.SortOrderNo);

            // 추가 정렬
            sortQuery = sortQuery.ThenSort(req.sort);

            return sortQuery;
        }
    }
}